﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using Microsoft.VirtualServer.Interop;
using System.Runtime.InteropServices;
using System.Threading;
using System.IO;
using System.Management;
using System.Diagnostics;

namespace AutomatingVirtualServer
{
    public partial class Form1 : Form
    {
        public Form1()
        {
            InitializeComponent();
            LoadVirtualMachines();
        }

        private void LoadVirtualMachines()
        {
            VMVirtualServer virtualServer = new VMVirtualServer();

            groupBoxVirtualMachines.Text =
                "Virtual Machines: " + 
                virtualServer.VirtualMachines.Count.ToString() +
                System.Environment.NewLine;

            foreach (VMVirtualMachine virtualMachine in
                virtualServer.VirtualMachines)
            {
                listBoxVirtualMachines.Items.Add(virtualMachine.Name);
                if (virtualMachine.Name == "Windows XP Pro SP2")
                    listBoxVirtualMachines.SelectedIndex =
                        listBoxVirtualMachines.Items.Count - 1;
            }
        }

        private string GetSelectedVirtualMachineName()
        {
            return listBoxVirtualMachines.SelectedItem.ToString();
        }

        private void button1_Click(object sender, EventArgs e)
        {
            VMVirtualServer virtualServer = new VMVirtualServer();

            VMVirtualMachine virtualMachine = 
                virtualServer.FindVirtualMachine(
                GetSelectedVirtualMachineName());

            if (virtualMachine == null)
                MessageBox.Show(String.Format("Virtual Machine ({0}) not found", GetSelectedVirtualMachineName()));
            else
                virtualMachine.Startup();
        }

        private void Progress(string text)
        {
            textBoxProgress.Text += text + System.Environment.NewLine;
            Application.DoEvents();
        }
        private void Progress(string text, bool newLine)
        {
            if (newLine)
                textBoxProgress.Text += text + System.Environment.NewLine;
            else
                textBoxProgress.Text += text;

            Application.DoEvents();
        }

        private VMVirtualMachine GetSelectedVirtualMachine()
        {
            VMVirtualServer virtualServer = new VMVirtualServer();
            VMVirtualMachine virtualMachine =
                virtualServer.FindVirtualMachine(
                GetSelectedVirtualMachineName());

            if (virtualMachine == null)
                MessageBox.Show(String.Format("Virtual Machine ({0}) not found", GetSelectedVirtualMachineName()));

            return virtualMachine;
        }

        private void button2_Click(object sender, EventArgs e)
        {
            VMVirtualMachine virtualMachine = 
                GetSelectedVirtualMachine();

            Progress(String.Format(
                "Virtual Machine State ({0}): {1}",
                virtualMachine.Name, virtualMachine.State.ToString()));
        }

        private void button3_Click(object sender, EventArgs e)
        {
            VMVirtualMachine virtualMachine =
                GetSelectedVirtualMachine();

            virtualMachine.TurnOff();
        }

        private void button4_Click(object sender, EventArgs e)
        {
            VMVirtualMachine virtualMachine =
                GetSelectedVirtualMachine();

            virtualMachine.GuestOS.Shutdown();
        }

        private void button5_Click(object sender, EventArgs e)
        {
            VMVirtualMachine virtualMachine =
                GetSelectedVirtualMachine();

            if (virtualMachine.GuestOS.IsHeartbeating)
                Progress("Heart is beating");
            else
                Progress("Heart is not beating");
        }

        private bool IsHeartbeating(VMVirtualMachine virtualMachine)
        {
            const int VM_E_ADDITIONS_NOT_AVAIL = -1610349308;
            try
            {
                // System.Runtime.InteropServices.COMException was unhandled
                // Message="Exception from HRESULT: 0xA0040504"
                // ErrorCode=-1610349308
                return virtualMachine.GuestOS.IsHeartbeating;
            }
            catch (COMException exception)
            {
                if (exception.ErrorCode == VM_E_ADDITIONS_NOT_AVAIL)
                    // Additions are not installed in this virtual
                    // machine, or the virtual machine has never
                    // been started.
                    return false;

                throw;
            }
        }

        private void button6_Click(object sender, EventArgs e)
        {
            VMVirtualMachine virtualMachine =
                GetSelectedVirtualMachine();

            if (IsHeartbeating(virtualMachine))
                Progress("Heart is beating");
            else
                Progress("Heart is not beating");
        }

        private string password;

        private string GetPassword()
        {
            // This opens up a file that will probably not be present
            // on your machine. The file contains the password to
            // my virtual machine. The actual value is useless to you
            // because you don't have my virtual machine.
            // You should replace this method with something that
            // returns the password for your virtual machine.
            if (password == null)
            {
                using (StreamReader streamReader = new StreamReader(
                    new FileStream(@"c:\temp\Oregan.txt",
                        FileMode.Open, FileAccess.Read)))
                {
                    string line = streamReader.ReadLine();
                    int index = line.IndexOf(" ");
                    password = line.Substring(0, index);
                    streamReader.Close();
                }
            }
            return password;
        }

        private void button7_Click(object sender, EventArgs e)
        {
            VMVirtualMachine virtualMachine =
                GetSelectedVirtualMachine();

            virtualMachine.Startup();
            Thread.Sleep(35000);
            virtualMachine.Keyboard.TypeAsciiText(GetPassword());
            virtualMachine.Keyboard.TypeKeySequence("Key_Enter");
        }

        private void button8_Click(object sender, EventArgs e)
        {
            VMVirtualServer virtualServer = new VMVirtualServer();

            string virtualHardDiskName =
                textBoxVirtualHardDiskFilename.Text;

            try
            {
                VMHardDisk virtualHardDisk =
                    virtualServer.GetHardDisk(virtualHardDiskName);

                Progress("Virtual hard disk already exists");
            }
            catch (FileNotFoundException)
            {
                VMTask task =
                    virtualServer.CreateDynamicVirtualHardDisk(
                    virtualHardDiskName, 20);

                task.WaitForCompletion(5000);
                Progress("Done.");
            }
        }

        private void button9_Click(object sender, EventArgs e)
        {
            VMVirtualMachine virtualMachine =
                GetSelectedVirtualMachine();

            string virtualHardDiskName =
                textBoxVirtualHardDiskFilename.Text;

            VMHardDiskConnection hardDiskConnection =
                virtualMachine.AddHardDiskConnection(
                virtualHardDiskName,
                VMDriveBusType.vmDriveBusType_IDE, 1, 1);

            Progress("Hard disk connected");
        }

        private void button13_Click(object sender, EventArgs e)
        {
            string vhdMountFilename =
                System.Environment.GetFolderPath(
                Environment.SpecialFolder.ProgramFiles) + 
                @"\Microsoft Virtual Server\VHDMount\VHDMount.exe";

            string vhdMountParameters =
                String.Format(@"/m /f ""{0}"" z",
                textBoxVirtualHardDiskFilename.Text);

            Process.Start(vhdMountFilename, vhdMountParameters);
        }

        private void button14_Click(object sender, EventArgs e)
        {
            string vhdMountFilename =
                System.Environment.GetFolderPath(
                Environment.SpecialFolder.ProgramFiles) +
                @"\Microsoft Virtual Server\VHDMount\VHDMount.exe";

            string vhdMountParameters =
                String.Format(@"/u ""{0}""",
                textBoxVirtualHardDiskFilename.Text);

            Process.Start(vhdMountFilename, vhdMountParameters);
        }

        private void button15_Click(object sender, EventArgs e)
        {
            VMVirtualMachine virtualMachine =
                GetSelectedVirtualMachine();

            virtualMachine.Keyboard.TypeKeySequence("DOWN,Key_LeftWindows, DOWN,Key_R, UP,Key_R, UP,Key_LeftWindows");
            Thread.Sleep(1000);
            string command = @"e:\nunit-console\nunit-console e:\UnitTests\TestLibraryTests.dll /xml:""e:\UnitTestResults\" + virtualMachine.Name + @".xml""";
            virtualMachine.Keyboard.TypeAsciiText(command);
            virtualMachine.Keyboard.TypeKeySequence("Key_Enter");
        }

        private uint scheduledJobId = 0;

        private uint CreateScheduledJob(
            string managementPath, string command,
            DateTime startDateTime, out uint scheduledJobId)
        {
            string startDateTimeString = ToDMTFTime(startDateTime);

            ManagementClass management = new ManagementClass(managementPath, "Win32_ScheduledJob", null);
            // "The RPC server is unavailable"
            // Check the firewall on the guest operating system is not blocking RPC
            // (On the guest OS run "netsh firewall set service RemoteAdmin").
            ManagementBaseObject methodParameters = management.GetMethodParameters("Create");
            methodParameters["Command"] = command;
            methodParameters["DaysOfMonth"] = 0;
            methodParameters["DaysOfWeek"] = 64 + 32 + 16 + 8 + 4 + 2 + 1;
            methodParameters["InteractWithDesktop"] = true;
            methodParameters["RunRepeatedly"] = false;
            methodParameters["StartTime"] = startDateTimeString;
            // use late binding to invoke "Create" method on "Win32_ScheduledJob" WMI class
            ManagementBaseObject outParams = management.InvokeMethod("Create", methodParameters, null);
            scheduledJobId = ((uint)(outParams.Properties["JobId"].Value));
            return ((uint)(outParams.Properties["ReturnValue"].Value));
        }

        public static string ToDMTFTime(DateTime dateParam)
        {
            string tempString = dateParam.ToString("********HHmmss.ffffff");
            TimeSpan tickOffset = TimeZone.CurrentTimeZone.GetUtcOffset(dateParam);
            tempString += (tickOffset.Ticks >= 0) ? '+' : '-';
            tempString += (Math.Abs(tickOffset.Ticks) / System.TimeSpan.TicksPerMinute).ToString("d3");
            return tempString;
        }

        private uint DeleteScheduledJob(string managementPath, uint scheduledJobId)
        {
            ManagementObject mo = new ManagementObject(managementPath);
            mo.Path.RelativePath = "Win32_ScheduledJob.JobId=" + "\"" + scheduledJobId + "\"";
            ManagementBaseObject inParams = null;
            // use late binding to invoke "Delete" method on "Win32_ScheduledJob" WMI class
            ManagementBaseObject outParams = mo.InvokeMethod("Delete", inParams, null);
            return ((uint)(outParams.Properties["ReturnValue"].Value));
        }

        private void button10_Click(object sender, EventArgs e)
        {
            VMVirtualMachine virtualMachine =
                GetSelectedVirtualMachine();

            string virtualMachineComputerName =
                ((IVMGuestOS2)virtualMachine.GuestOS).ComputerName;

            string managementPath = String.Format(
                @"\\{0}\root\cimv2", virtualMachineComputerName);

            string command = @"c:\temp\RunTests.bat";

            uint success = CreateScheduledJob(
                managementPath, command, DateTime.Now.AddMinutes(1),
                out scheduledJobId);

            if (success == 0)
                Progress("Scheduled Task created");
            else
                Progress("Scheduled Task not created");
        }

        private void button11_Click(object sender, EventArgs e)
        {
            if (scheduledJobId != 0)
            {
                VMVirtualMachine virtualMachine =
                    GetSelectedVirtualMachine();

                string virtualMachineComputerName =
                    ((IVMGuestOS2)virtualMachine.GuestOS).ComputerName;

                string managementPath = String.Format(
                    @"\\{0}\root\cimv2", virtualMachineComputerName);

                DeleteScheduledJob(managementPath, scheduledJobId);

                scheduledJobId = 0;

                Progress("Scheduled job deleted");
            }
            else
                Progress("There is no scheduled job to delete");
        }

        private void button12_Click(object sender, EventArgs e)
        {
            VMVirtualMachine virtualMachine =
                GetSelectedVirtualMachine();

            Progress("Starting the virtual machine...");
            virtualMachine.Startup();
            Thread.Sleep(35000);

            Progress("Logging on...");
            virtualMachine.Keyboard.TypeAsciiText(GetPassword());
            virtualMachine.Keyboard.TypeKeySequence("Key_Enter");

            Progress("Waiting for heart beat...");
            while (!IsHeartbeating(virtualMachine))
            {
                Thread.Sleep(500);
            }

            Progress("Waiting for WMI to initialize...");
            Thread.Sleep(10000);

            Progress("Scheduling a job to run the unit tests...");
            string virtualMachineComputerName =
                ((IVMGuestOS2)virtualMachine.GuestOS).ComputerName;

            string managementPath = String.Format(
                @"\\{0}\root\cimv2", virtualMachineComputerName);

            string command = @"c:\temp\RunTests.bat";
            uint success = CreateScheduledJob(
                managementPath, command, DateTime.Now.AddMinutes(1), out scheduledJobId);
            if (success == 0)
            {
                // wait for the scheduled job to start (1 minute) and then wait for it to complete (10 seconds)
                Progress("Waiting for unit tests to run...");
                Thread.Sleep(60000 + 10000);

                Progress("Deleting scheduled job...");
                DeleteScheduledJob(managementPath, scheduledJobId);
                scheduledJobId = 0;
            }
            else
                Progress("Failed to schedule job");

            Progress("Shutting down the virtual machine...");
            VMTask task = virtualMachine.GuestOS.Shutdown();
            task.WaitForCompletion(30000);
            Progress("Done.");
        }

        private void button16_Click(object sender, EventArgs e)
        {
            VMVirtualMachine virtualMachine =
                GetSelectedVirtualMachine();

            Progress("Starting the virtual machine...");
            virtualMachine.Startup();
            Thread.Sleep(35000);

            Progress("Logging on...");
            virtualMachine.Keyboard.TypeAsciiText(GetPassword());
            virtualMachine.Keyboard.TypeKeySequence("Key_Enter");

            Progress("Waiting for heart beat...");
            while (!IsHeartbeating(virtualMachine))
            {
                Thread.Sleep(500);
            }

            Thread.Sleep(1000);

            Progress("Running the unit tests...");
            virtualMachine.Keyboard.TypeKeySequence("DOWN,Key_LeftWindows, DOWN,Key_R, UP,Key_LeftWindows, UP,Key_R");
            Thread.Sleep(1000);
            string command = @"e:\nunit-console\nunit-console e:\UnitTests\TestLibraryTests.dll /xml:""e:\UnitTestResults\" + virtualMachine.Name + @".xml""";
            virtualMachine.Keyboard.TypeAsciiText(command);
            virtualMachine.Keyboard.TypeKeySequence("Key_Enter");

            Progress("Waiting for unit tests to run...");
            Thread.Sleep(15000);

            Progress("Shutting down the virtual machine...");
            VMTask task = virtualMachine.GuestOS.Shutdown();
            task.WaitForCompletion(30000);
            Progress("Done.");
        }
    }
}
